package hdm.mi.growbros.service;

import hdm.mi.growbros.exceptions.PlantNotFoundException;
import hdm.mi.growbros.models.GardenEntry;
import hdm.mi.growbros.models.plant.Plant;
import hdm.mi.growbros.models.user.User;
import hdm.mi.growbros.repositories.GardenRepository;
import hdm.mi.growbros.repositories.PlantRepository;
import hdm.mi.growbros.util.DateTimeUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;

@Service
@RequiredArgsConstructor
public class GardenService {
    private final GardenRepository gardenRepository;
    private final PlantRepository plantRepository;

    public List<Plant> getUserPlants(User user, String sort) {
        if (sort == null) return getAllPlantsOrderedByName(user);
        return switch (sort) {
            case "createdAt" -> getAllPlantsOrderedByCreatedDate(user);
            case "plantDate" -> getAllPlantsOrderedByNextPossiblePlantDate(user);
            case "harvestDate" -> getAllPlantsOrderedByNextPossibleHarvest(user);
            default -> getAllPlantsOrderedByName(user);
        };
    }

    public Plant addPlantToGarden(Long plantId, User user) {
        //get user
        Plant plant = plantRepository.findById(plantId).orElseThrow(() -> new PlantNotFoundException(plantId));
        GardenEntry gardenEntry = new GardenEntry();
        gardenEntry.setPlant(plant);
        gardenEntry.setUser(user);
        var result = gardenRepository.save(gardenEntry);
        return result.getPlant();
    }

    public void removeFromGarden(Long entryId, User authenticatedUser) {
        gardenRepository.deleteByIdAndUser(entryId, authenticatedUser);
    }

    public int clearGarden(User authenticatedUser) {
        return gardenRepository.deleteAllByUser(authenticatedUser);
    }

    private List<Plant> getAllPlantsOrderedByName(User user) {
        return mapGardenEntriesToPlants(
                gardenRepository.findByUser(user, Sort.by("plant.name"))
        );
    }

    private List<Plant> getAllPlantsOrderedByCreatedDate(User user) {
        return mapGardenEntriesToPlants(
                gardenRepository.findByUser(user, Sort.by("createdDate"))
        );
    }

    private List<Plant> getAllPlantsOrderedByNextPossiblePlantDate(User user) {
        return mapGardenEntriesToPlants(
                gardenRepository.findAllByNearestPlantingWeek(user, DateTimeUtils.getCurrentWeekForToday())
        );
    }

    private List<Plant> getAllPlantsOrderedByNextPossibleHarvest(User user) {
        return mapGardenEntriesToPlants(
                gardenRepository.findAllByNearestHarvestWeek(user, DateTimeUtils.getCurrentWeekForToday())
        );
    }

    private List<Plant> mapGardenEntriesToPlants(Collection<GardenEntry> entries) {
        return entries
                .stream()
                .map(GardenEntry::getPlant)
                .toList();
    }

    public int countByUser(User user) {
        return gardenRepository.countByUser(user);
    }
}
